@@ -22,7 +22,7 @@
 
     After half a decade, where in the meantime Taco also had turned MetaPost into a library, we
     had a version that had proved itself well. The following years, with Taco having less time
-    available, I started loking at the code. Some more got added to the Lua interfaces. Math got
+    available, I started looking at the code. Some more got added to the Lua interfaces. Math got
     split code paths and some new primitives were introduced. Luigi started taking care of managing
     the code base so that I could cross compile for \MSWINDOWS. He also deals with the libraries
     that were used and integration in \TEXLIVE\ and maintains the (by now stable) \METAPOST\ code
@@ -54,9 +54,9 @@
     Over the 15 years that it took to go from the first version of \LUATEX\ in 2005 to the first
     release of \LUAMETATEX\ in 2020 (although intermediate versions have always been good enough
     to be used in production with \CONTEXT) I've written numerous articles in user group journals
-    as well as several presentations each year on progress and features. There are also wrapups
+    as well as several presentations each year on progress and features. There are also wrap-ups
     available in the \CONTEXT\ distribution that shed some light on how the developments
-    progress(ed). In the end it's all a work of many. There are no commercial interrests and
+    progress(ed). In the end it's all a work of many. There are no commercial interests and
     everything is done out of love for TeX and in free time, so take that into account when you
     bark about code or documentation.
 
@@ -69,7 +69,7 @@
     engine takes the 200 range. Revisions range from 00 upto 99 and the dates \unknown\ depend on
     the mood. The |2.05.00| version with the development id |20200229| was more or less the first
     official version, in the sense that most of the things on my initial todo list were done. It's
-    a kind of virtual date as it happens to be a leapyear. As with LuaTeX the .10 version will be
+    a kind of virtual date as it happens to be a leap year. As with LuaTeX the .10 version will be
     the first 'stable' one, released somewhere around the ConTeXt 2021 meeting.
 
     2.08.18 : around TeXLive 2021 code freeze (so a bit of a reference version)
@@ -80,27 +80,37 @@
     2.10.05 : late January 2023 
     2.10.07 : around TeXLive 2023
     2.10.08 : close to BachoTeX 2023 
-    2.10.09 : evolved in May/June 2023 (math & text linebreak experiments) 
+    2.10.09 : evolved in May/June 2023 (math & text line break experiments) 
     2.10.10 : around the ctx 2023 meeting 
-    2.10.11 : till the end of 2024 
+    2.10.11 : first half of 2024
+    2.10.12 : around the ctx 2024 meeting
+    2.11.07 : just after texlive 2025 code freeze
+    2.11.08 : just before the ctx 2025 meeting
 
     At some point the \CONTEXT\ group will be responsible for guaranteeing that the official version
-    is what comes with \CONTEXT\ and that long term support and stabilty is guaranteed and that no 
+    is what comes with \CONTEXT\ and that long term support and stability is guaranteed and that no 
     feature creep or messing up happens. We'll see. 
 
     Hans Hagen
 
+    PS. Does anyone use (lib)cerf? If so, should I strip it a bit? 
+    PS. Maybe use the Lua randomizer.
+    PS. Should we stay with mimalloc (too different platform code?)
+
 */
 
 # include "tex/textypes.h"
 
-# define luametatex_version          2
-# define luametatex_revision         11
-# define luametatex_release          02
-# define luametatex_version_string   "2.11.02"
-# define luametatex_version_number   211.2
-# define luametatex_development_id   20240512
+/*tex Currently LUAC_FORMAT is set to 5 awaiting an official version bump. */
 
+# define luametatex_majorversion     2
+# define luametatex_minorversion     11
+# define luametatex_version          211
+# define luametatex_revision         0
+# define luametatex_release          8
+# define luametatex_version_string   "2.11.08"
+# define luametatex_version_number   211.8
+# define luametatex_development_id   20250917
 # define luametatex_name_camelcase   "LuaMetaTeX"
 # define luametatex_name_lowercase   "luametatex"
 # define luametatex_copyright_holder "Taco Hoekwater, Hans Hagen, Wolfgang Schuster & Mikael Sundqvist"
@@ -119,6 +129,8 @@
 */
 
 typedef struct version_state_info {
+    int         majorversion;
+    int         minorversion; 
     int         version;
     int         revision;
     int         release; 
@@ -131,6 +143,7 @@ typedef struct version_state_info {
     int         luaversionmajor;
     int         luaversionminor;
     int         luaversionrelease;
+    int         luaformat;
     double      luatexversion;
     double      luaversion;
 } version_state_info;
@@ -139,7 +152,7 @@ extern version_state_info lmt_version_state;
 
 /*tex
 
-    This is actually the main headere file. Of course we could split it up and be more explicit in
+    This is actually the main header file. Of course we could split it up and be more explicit in
     other files but this is simple and just works. There is of course some overhead in loading
     headers that are not used, but because compilation is simple and fast I don't care.
 
@@ -190,6 +203,7 @@ extern version_state_info lmt_version_state;
 
 # include "lua.h"
 # include "lauxlib.h"
+# include "lundump.h"
 
 # define LUA_VERSION_STRING ("Lua " LUA_VERSION_MAJOR "." LUA_VERSION_MINOR "." LUA_VERSION_RELEASE)
 
@@ -266,8 +280,12 @@ extern version_state_info lmt_version_state;
 /*tex This is not used (yet) as I don't expect much from it, but \LUA\ has some of it. */
 
 # if defined(__GNUC__)
-#   define lmt_likely(x)   (__builtin_expect(((x) != 0), 1))
-#   define lmt_unlikely(x) (__builtin_expect(((x) != 0), 0))
+    // Lua: 
+ // # define lmt_likely(x)   (__builtin_expect(((x) != 0), 1))
+ // # define lmt_unlikely(x) (__builtin_expect(((x) != 0), 0))
+    // Kernel: 
+    # define lmt_likely(x)   (__builtin_expect(!!(x), 1))
+    # define lmt_unlikely(x) (__builtin_expect(!!(x), 0))
 # else
 #   define lmt_likely(x)   (x)
 #   define lmt_unlikely(x) (x)
@@ -276,6 +294,7 @@ extern version_state_info lmt_version_state;
 # include "utilities/auxarithmetic.h"
 # include "utilities/auxmemory.h"
 # include "utilities/auxposit.h"
+# include "utilities/auxbytemaps.h"
 # include "utilities/auxzlib.h"
 
 # include "tex/texmainbody.h"
@@ -296,9 +315,9 @@ extern version_state_info lmt_version_state;
 # include "tex/texconditional.h"
 # include "tex/textextcodes.h"
 # include "tex/texmathcodes.h"
+# include "tex/texsnapping.h"
 # include "tex/texalign.h"
 # include "tex/texrules.h"
-/*        "tex/texdirections.h" */
 # include "tex/texerrors.h"
 # include "tex/texinputstack.h"
 # include "tex/texstringpool.h"
@@ -315,9 +334,10 @@ extern version_state_info lmt_version_state;
 # include "tex/texbuildpage.h"
 # include "tex/texmaincontrol.h"
 # include "tex/texdumpdata.h"
-# include "tex/texmainbody.h"
 # include "tex/texnodes.h"
+# include "tex/texspecifications.h"
 # include "tex/texdirections.h"
+# include "tex/texdiscretionaries.h"
 # include "tex/texlinebreak.h"
 # include "tex/texmath.h"
 # include "tex/texmlist.h"
@@ -325,8 +345,8 @@ extern version_state_info lmt_version_state;
 # include "tex/texprimitive.h"
 # include "tex/texequivalents.h"
 # include "tex/texfont.h"
+# include "tex/texbalance.h"
 # include "tex/texlanguage.h"
-
 # include "lua/lmtcallbacklib.h"
 # include "lua/lmttokenlib.h"
 # include "lua/lmtnodelib.h"
@@ -336,6 +356,9 @@ extern version_state_info lmt_version_state;
 # include "lua/lmtluaclib.h"
 # include "lua/lmttexlib.h"
 # include "lua/lmtenginelib.h"
+# include "luarest/lmtvectorlib.h"
+# include "luarest/lmteffectslib.h"
+# include "luarest/lmtbytemaplib.h"
 
 /*tex
 
